@@ -1,5 +1,6 @@
 /* MI Command Set - disassemble commands.
-   Copyright (C) 2000, 2001, 2002, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2001, 2002, 2007, 2008, 2009, 2010, 2011
+   Free Software Foundation, Inc.
    Contributed by Cygnus Solutions (a Red Hat company).
 
    This file is part of GDB.
@@ -18,6 +19,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
+#include "arch-utils.h"
 #include "target.h"
 #include "value.h"
 #include "mi-cmds.h"
@@ -38,20 +40,25 @@
    FILENAME: The name of the file where we want disassemble from.
    LINE: The line around which we want to disassemble. It will
    disassemble the function that contins that line.
-   HOW_MANY: Number of disassembly lines to display. In mixed mode, it
+   HOW_MANY: Number of disassembly lines to display. With source, it
    is the number of disassembly lines only, not counting the source
    lines.  
 
    always required:
 
-   MODE: 0 or 1 for disassembly only, or mixed source and disassembly,
-   respectively. */
+   MODE: 0 -- disassembly.
+         1 -- disassembly and source.
+         2 -- disassembly and opcodes.
+         3 -- disassembly, source and opcodes.
+*/
 void
 mi_cmd_disassemble (char *command, char **argv, int argc)
 {
+  struct gdbarch *gdbarch = get_current_arch ();
+  struct ui_out *uiout = current_uiout;
   CORE_ADDR start;
 
-  int mixed_source_and_assembly;
+  int mode, disasm_flags;
   struct symtab *s;
 
   /* Which options have we processed ... */
@@ -67,6 +74,7 @@ mi_cmd_disassemble (char *command, char **argv, int argc)
   int how_many = -1;
   CORE_ADDR low = 0;
   CORE_ADDR high = 0;
+  struct cleanup *cleanups = make_cleanup (null_cleanup, NULL);
 
   /* Options processing stuff. */
   int optind = 0;
@@ -75,7 +83,7 @@ mi_cmd_disassemble (char *command, char **argv, int argc)
   {
     FILE_OPT, LINE_OPT, NUM_OPT, START_OPT, END_OPT
   };
-  static struct mi_opt opts[] = {
+  static const struct mi_opt opts[] = {
     {"f", FILE_OPT, 1},
     {"l", LINE_OPT, 1},
     {"n", NUM_OPT, 1},
@@ -88,7 +96,7 @@ mi_cmd_disassemble (char *command, char **argv, int argc)
      encountered. */
   while (1)
     {
-      int opt = mi_getopt ("mi_cmd_disassemble", argc, argv, opts,
+      int opt = mi_getopt ("-data-disassemble", argc, argv, opts,
 			   &optind, &optarg);
       if (opt < 0)
 	break;
@@ -97,6 +105,7 @@ mi_cmd_disassemble (char *command, char **argv, int argc)
 	case FILE_OPT:
 	  file_string = xstrdup (optarg);
 	  file_seen = 1;
+	  make_cleanup (xfree, file_string);
 	  break;
 	case LINE_OPT:
 	  line_num = atoi (optarg);
@@ -125,17 +134,24 @@ mi_cmd_disassemble (char *command, char **argv, int argc)
   if (!((line_seen && file_seen && num_seen && !start_seen && !end_seen)
 	|| (line_seen && file_seen && !num_seen && !start_seen && !end_seen)
 	|| (!line_seen && !file_seen && !num_seen && start_seen && end_seen)))
-    error
-      ("mi_cmd_disassemble: Usage: ( [-f filename -l linenum [-n howmany]] | [-s startaddr -e endaddr]) [--] mixed_mode.");
+    error (_("-data-disassemble: Usage: ( [-f filename -l linenum [-n "
+	     "howmany]] | [-s startaddr -e endaddr]) [--] mode."));
 
   if (argc != 1)
-    error
-      ("mi_cmd_disassemble: Usage: [-f filename -l linenum [-n howmany]] [-s startaddr -e endaddr] [--] mixed_mode.");
+    error (_("-data-disassemble: Usage: [-f filename -l linenum "
+	     "[-n howmany]] [-s startaddr -e endaddr] [--] mode."));
 
-  mixed_source_and_assembly = atoi (argv[0]);
-  if ((mixed_source_and_assembly != 0) && (mixed_source_and_assembly != 1))
-    error (_("mi_cmd_disassemble: Mixed_mode argument must be 0 or 1."));
+  mode = atoi (argv[0]);
+  if (mode < 0 || mode > 3)
+    error (_("-data-disassemble: Mode argument must be 0, 1, 2, or 3."));
 
+  /* Convert the mode into a set of disassembly flags */
+
+  disasm_flags = 0;
+  if (mode & 0x1)
+    disasm_flags |= DISASSEMBLY_SOURCE;
+  if (mode & 0x2)
+    disasm_flags |= DISASSEMBLY_RAW_INSN;
 
   /* We must get the function beginning and end where line_num is
      contained. */
@@ -144,16 +160,18 @@ mi_cmd_disassemble (char *command, char **argv, int argc)
     {
       s = lookup_symtab (file_string);
       if (s == NULL)
-	error (_("mi_cmd_disassemble: Invalid filename."));
+	error (_("-data-disassemble: Invalid filename."));
       if (!find_line_pc (s, line_num, &start))
-	error (_("mi_cmd_disassemble: Invalid line number"));
+	error (_("-data-disassemble: Invalid line number"));
       if (find_pc_partial_function (start, NULL, &low, &high) == 0)
-	error (_("mi_cmd_disassemble: No function contains specified address"));
+	error (_("-data-disassemble: "
+		 "No function contains specified address"));
     }
 
-  gdb_disassembly (uiout,
+  gdb_disassembly (gdbarch, uiout,
   		   file_string,
-		   line_num,
-		   mixed_source_and_assembly, how_many, low, high);
+  		   disasm_flags,
+		   how_many, low, high);
 
+  do_cleanups (cleanups);
 }
