@@ -1,6 +1,6 @@
 /* Objective-C language support definitions for GDB, the GNU debugger.
 
-   Copyright (C) 1992, 2005, 2007, 2008, 2009, 2010
+   Copyright (C) 1992, 2005, 2007, 2008, 2009, 2010, 2011
    Free Software Foundation, Inc.
 
    Contributed by Apple Computer, Inc.
@@ -21,6 +21,8 @@
 #if !defined(OBJC_LANG_H)
 #define OBJC_LANG_H
 
+#include "cp-support.h"		/* For VEC (const_char_ptr) */
+
 struct stoken;
 
 struct value;
@@ -39,15 +41,7 @@ extern char *objc_demangle (const char *mangled, int options);
 
 extern int find_objc_msgcall (CORE_ADDR pc, CORE_ADDR *new_pc);
 
-extern char *parse_selector (char *method, char **selector);
-
-extern char *parse_method (char *method, char *type, 
-			   char **class, char **category, 
-			   char **selector);
-
-extern char *find_imps (struct symtab *symtab, struct block *block,
-			char *method, struct symbol **syms, 
-			unsigned int *nsym, unsigned int *ndebug);
+extern char *find_imps (char *method, VEC (const_char_ptr) **symbol_names);
 
 extern struct value *value_nsstring (struct gdbarch *gdbarch,
 				     char *ptr, int len);
