@@ -1,4 +1,5 @@
-/* Copyright 2003, 2004, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+/* Copyright 2003, 2004, 2007, 2008, 2009, 2010, 2011
+Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,7 +22,17 @@ namespace C
   class OtherFileClass {
   public:
     int z;
+
+    typedef short cOtherFileClassType;
+    typedef long cOtherFileClassType2;
+    static const cOtherFileClassType cOtherFileClassVar = 318;
+    static const cOtherFileClassType2 cOtherFileClassVar2 = 320;
+    cOtherFileClassType cOtherFileClassVar_use ();
   };
+  OtherFileClass::cOtherFileClassType OtherFileClass::cOtherFileClassVar_use ()
+  {
+    return cOtherFileClassVar + cOtherFileClassVar2;
+  }
 
   namespace {
     int cXOtherFile = 29;
@@ -35,6 +46,15 @@ namespace C
     static OtherFileClass *c = new OtherFileClass();
     c->z = cOtherFile + cXOtherFile;
   }
+
+  typedef short cOtherFileType;
+  typedef long cOtherFileType2;
+  static const cOtherFileType cOtherFileVar = 319;
+  static const cOtherFileType2 cOtherFileVar2 = 321;
+  cOtherFileType cOtherFileVar_use ()
+  {
+    return cOtherFileVar + cOtherFileVar2;
+  }
 }
 
 namespace {
